/*******************************************************************************
Demo project for Olimex board: MOD-LCD 4.3" RevA

In order to work correctly you must:

0)  Connect USB serial cables in the respective UEXT pins as follow:
    2-GND (blue wire); 3-TX (green wire); 4-RX (red wire).
    UART must be configured:
    BaudRate: 115200
    DataBits: 8
    StopBits: 1
    Parity:   None

1)  SDCard formated or containing some files/folders.

2)  USB-A Mini to the USB-OTG connector so you can browse the SDCard via
    Windows Explorer.

3)  JLink to JTag connector if you intend to program or debug it.
    Otherwise if you wish to use the project standalone you don't need the JLink.

Brief description:
On Power-ON/Reset on the console terminal will be shown a welcome message.
After that there will be something similar to a command prompt where you can
send several commands. You can see the list of command if you send command:
  'help all'
For more details of a specific command type help <cmd>
For example: 'help files'

As for the rest of the peripherals:
When you push the button the yellow LED is ON and otherwise the LED is OFF.
When you touch the screen on the console will be shown the coordinates
of the point where you touch it (if it's only one position, otherwise the
results will be incorrect).

Author:   Stanimir Petev
Company:  OLIMEX
Date:     12.07.2012

*******************************************************************************/

#include "includes.h"

#define MAX_NUMBER_PICTURES   20
#define TIMER0_TICK_PER_SEC   10
#define DLY_100US             500

extern Int32U SDRAM_BASE_ADDR;
volatile Boolean TickFlag;


#pragma location="USB_DMA_RAM"
#pragma data_alignment=4
__no_init Int8U Lun0Buffer[512];


// SPP +
#pragma section = "SDRAM_VRAM"
#pragma location = "SDRAM_VRAM"
EmbeddedFileSystem efs;
#pragma section = "SDRAM_VRAM"
#pragma location = "SDRAM_VRAM"
EmbeddedFile file_r, file_w;
#pragma section = "SDRAM_VRAM"
#pragma location = "SDRAM_VRAM"
DirList MyList;
char GlobalPath[MAX_PATH_LENGTH] = "/";   // root dir

pPic_t Displayed_Picture = NULL;
pPic_t *Picture;
int PicIndex=0, PicCount, SDCardError = 0, BaudRateKey=3, Touch_Threshold=5;

pPic_t Load_Bitmap_to_Memory (char Name[]);
void Load_All_Bitmap_to_Memory (void);
// SPP -

/*************************************************************************
 * Function Name: Timer0IntrHandler
 * Parameters: none
 *
 * Return: none
 *
 * Description: Timer 0 interrupt handler
 *
 *************************************************************************/
void Timer0IntrHandler (void)
{
  TickFlag = TRUE;
  // clear interrupt
  T0IR_bit.MR0INT = 1;
  VICADDRESS = 0;
}

void InitRTC(void)
{
	PCONP_bit.PCRTC = 1;
	
	CCR_bit.CTCRST = 1;
	CCR_bit.CTCRST = 0;
	CCR_bit.CTTEST = 0;
	CCR_bit.CLKSRC = 1;
	CCR_bit.CLKEN = 1;
}

/*************************************************************************
 * Function Name: DelayResolution100us
 * Parameters: Int32U Dly
 *
 * Return: none
 *
 * Description: Delay ~ (arg * 100us)
 *
 *************************************************************************/
void DelayResolution100us(Int32U Dly)
{
  for(; Dly; Dly--)
  {
    for(volatile Int32U j = DLY_100US; j; j--)
    {
    }
  }
}

void Initialize (void)
{
// Turn Off the display
  GLCD_Ctrl (FALSE);

  // Init GPIO
  GpioInit();

  // initialize button
  BUT1_FDIR = ~BUT1_MASK;

#ifndef SDRAM_DEBUG
  // MAM init
  MAMCR_bit.MODECTRL = 0;
  MAMTIM_bit.CYCLES  = 3;   // FCLK > 40 MHz
  MAMCR_bit.MODECTRL = 2;   // MAM functions fully enabled
  // Init clock
  InitClock();
  // SDRAM Init
  SDRAM_Init();
#endif // SDRAM_DEBUG

  // Init VIC
  VIC_Init();

  // GLCD init
  GLCD_Init (NULL, NULL);

  // Cursor init
  GLCD_Cursor_Dis(0);
  GLCD_Copy_Cursor ((Int32U *)Cursor, 0, sizeof(Cursor)/sizeof(Int32U));
  GLCD_Cursor_Cfg(CRSR_FRAME_SYNC | CRSR_PIX_32);
  GLCD_Move_Cursor((C_GLCD_H_SIZE - CURSOR_H_SIZE)/2, (C_GLCD_V_SIZE - CURSOR_V_SIZE)/2);
  GLCD_Cursor_En(0);

  // Init USB Link  LED
  USB_D_LINK_LED_FDIR = USB_D_LINK_LED_MASK;
  USB_D_LINK_LED_FSET = USB_D_LINK_LED_MASK;

#if 1
  // Enable TIM0 clocks
  PCONP_bit.PCTIM0 = 1; // enable clock
  // Init Time0
  T0TCR_bit.CE = 0;     // counting  disable
  T0TCR_bit.CR = 1;     // set reset
  T0TCR_bit.CR = 0;     // release reset
  T0CTCR_bit.CTM = 0;   // Timer Mode: every rising PCLK edge
  T0MCR_bit.MR0I = 1;   // Enable Interrupt on MR0
  T0MCR_bit.MR0R = 1;   // Enable reset on MR0
  T0MCR_bit.MR0S = 0;   // Disable stop on MR0
  // set timer 0 period
  T0PR = 0;
  T0MR0 = SYS_GetFpclk(TIMER0_PCLK_OFFSET)/(TIMER0_TICK_PER_SEC);
  // init timer 0 interrupt
  T0IR_bit.MR0INT = 1;  // clear pending interrupt
  VIC_SetVectoredIRQ(Timer0IntrHandler,0,VIC_TIMER0);
  VICINTENABLE |= 1UL << VIC_TIMER0;
  T0TCR_bit.CE = 1;     // counting Enable
#endif
  
  // Init touch screen
  TouchScrInit();

  // Init RTC
  InitRTC();

  // Init SCSI module
  ScsiInit();

#ifdef  USE_UART
  // Init VIC
  //VIC_Init();   // this function is already called in this module
  // Init UART 0
  UartInit(UART_0,4,NORM);
  // Configurate Uart 0
  UART_Configurate (115200);
#endif


  __enable_interrupt();
  
  // LUNs Init
  LunInit(SD_DISK_LUN,SdDiskInit,SdGetDiskCtrlBkl,SdDiskIO);

  // Init SD card driver
  // SCSI init push init SD driver message for an each LUN
  for(Int32U i = 0; i < SCSI_LUN_NUMB; i++)
  {
    // Implement LUNs messages
    if(LunImp(i))
    {
      USB_D_LINK_LED_FCLR = USB_D_LINK_LED_MASK;
    }
    else
    {
      USB_D_LINK_LED_FSET = USB_D_LINK_LED_MASK;
    }
  }

  // Soft connection enable
  USB_ConnectRes(TRUE);

  // RTC indication
  USB_D_LINK_LED_SEL = 0; // GPIO
  USB_D_LINK_LED_FSET = USB_D_LINK_LED_MASK;
  USB_D_LINK_LED_FDIR |= USB_D_LINK_LED_MASK;

   
  // Init SD card file system
  if (efs_init (&efs, 0))   // Init File system
  {
    SDCardError = 1;
//    Error();
  }
  if (ls_openDir (&MyList, &efs.myFs, GlobalPath))   // Init root dir
  {
    SDCardError = 1;
//    Error();
  }
  
  Synchronize_Buffers ();
  
  // Turn ON the display 
  GLCD_Ctrl (TRUE);
  
  return ;
}

void Display_New_Picture (int New, Point X, Point Y, int Mask, int ClearFlag)
{
  // Display the picture on the display
  PicIndex = (PicIndex + New + PicCount) % PicCount;
  GLCD_ShowPic_in_Box (Picture[PicIndex], X, Y, Mask, ClearFlag);
  return;
}

void Load_All_Bitmap_to_Memory (void)
{
  MyFile FileInfo;
  char Success;
  Success = 0;
  do
  {
    // if the current folder contains more files/folders --> get the next one
    if (ls_getNext(&MyList)==0)
    {
      // forming the name of the file/folder
      // the structure of the FileInfo consist of:
      // Name (up to 8 characters), Extension (none for folders) and 
      // FullName (both of them combined and separated by '.')
      Form_File_Name (&FileInfo, MyList.currentEntry.FileName);

      // Filtering files. Visualize olny the *.bmp files
      // if any other type --> skip to the next file (if exist)
      if (My_StrCmp (FileInfo.Extension, "bmp", NO_CASE_SENS))
        continue;

      Picture[PicIndex++] = Load_Bitmap_to_Memory (FileInfo.FullName);
    }
    else  // if the current folder DOES NOT contain more files/folders --> reinit
    {
      if (ls_openDir (&MyList, &efs.myFs, GlobalPath))
        Error();
      Success = 1;
    }
  }
  while (!Success); // the purpose of this loop is to skip the non-bitmap files
  PicCount = PicIndex;
  PicIndex = 0;
  return;
}

pPic_t Load_Bitmap_to_Memory (char Name[])
{
  pPic_t pPic;
  unsigned long int size;
  unsigned char *bmp;
  unsigned char buf[54];
  char FullPathName[MAX_PATH_LENGTH + MAX_FILE_NAME_LENGTH + MAX_FILE_EXT_LENGTH];
  
  // check the current directory
  if (ls_openDir (&MyList, &efs.myFs, GlobalPath))
  {
    WriteLn ("Error while opening the folder");
    return NULL;
  }
  
  // put  the full path and name of the file in one string
  sprintf (FullPathName, "%s%s", GlobalPath, Name);
  
  // Open the file
  if (file_fopen (&file_r, &efs.myFs, FullPathName, 'r') != 0)
  {
    WriteLn ("Wrong file name!");
    return NULL;
  }

  // Read the header of the bitmap (first 54 bytes)
  file_read (&file_r, 54, buf);

  // Using the info in the header to calculate size of the bitmap
  size = (buf[2]<<0) + (buf[3]<<8) + (buf[4]<<16) + (buf[5]<<24);
  
  // Reset the pointer of the file
  if (file_fclose (&file_r))
    Error ();
  if (file_fopen (&file_r, &efs.myFs, FullPathName, 'r'))
    Error ();
      
  
  // Allocate enough heap-memory for the byte-array buffer to store the bitmap data
  bmp = (unsigned char*) malloc(size*sizeof(unsigned char));
  if (bmp == NULL)
    return NULL;

  // Load all the data (including the header) into a byte-array buffer
  file_read (&file_r, size, bmp);

  // Close the file
  file_fclose (&file_r);

  // Converts the byte-array into pixel-array (picture)
  pPic = GLCD_LoadPic((bmp_t *)bmp);

  // Free the allocated memory for the byte-array buffer
  free(bmp);
  
  return pPic;
}

void PrintGlobalPath (void)
{
  NewLine ();
  Write ("<root:");
  Write (GlobalPath);
  Write ("> ");
}

/*************************************************************************
 * Function Name: main
 * Parameters: none
 *
 * Return: none
 *
 * Description: main
 *
 *************************************************************************/

int main(void)
{
  int cursor_x = (C_GLCD_H_SIZE - CURSOR_H_SIZE)/2, cursor_y = (C_GLCD_V_SIZE - CURSOR_V_SIZE)/2;
  ToushRes_t XY_Touch;
  // SPP +
  int TouchScreenFlag=0;
  // UART input buffer
  char UART_Input[256];
  // Welcome message
  char Welcome_Message[] =  "\f"\
                            "*********************************\n\r"\
                            "*    OLIMEX - MOD LCD 4.3 demo   *\n\r"\
                            "* Use the console to manipulate *\n\r"\
                            "*   the pictures with figures   *\n\r"\
                            "*     type 'help all' to see    *\n\r"\
                            "*         all the options       *\n\r"\
                            "*     Note: The Console isn't   *\n\r"\
                            "*          case sensitive       *\n\r"\
                            "*********************************\n\r";
  // SPP -
  Initialize ();

  Write(Welcome_Message);
  if (SDCardError)
    WriteLn ("SD card problem detected!");
  PrintGlobalPath ();

  while(1)
  {
    if (!(BUT1_FIO & BUT1_MASK))                    // if Button is pushed
      USB_D_LINK_LED_FCLR = USB_D_LINK_LED_MASK;    // LED is ON
    else                                            // else
      USB_D_LINK_LED_FSET = USB_D_LINK_LED_MASK;    // LED is OFF

#ifdef  USE_UART
      if (ReadLn (UART_Input))
      {
        int i, argc;      // arguments count
        char **argv=NULL; // arguments values
        CommandKey Cmd;   // command type
        
        Cmd = Form_Command (UART_Input, &argc, &argv);
        Dispatch_Command (Cmd, argc, argv);
        
        // free memory for the arguments
        for (i=0; i<argc; i++)
          free (argv[i]);
        free (argv);
        PrintGlobalPath ();
      }
#endif

      // SD Card routine
      // Implement LUNs messages
      for(Int32U i = 0; i < SCSI_LUN_NUMB; i++)
      {
          LunImp(i);
      }

      // Touch screen routine
      if(TouchGet(&XY_Touch))
      {
          if( abs((int)XY_Touch.X - cursor_x) > Touch_Threshold || abs((int)XY_Touch.Y - cursor_y) > Touch_Threshold)
          {
            char Output[32];
            cursor_x = XY_Touch.X;
            cursor_y = XY_Touch.Y;
            if (TouchScreenFlag)
              Write ("Drag to position: ");
            else
            {
              NewLine ();
              Write ("Screen touched on position: ");
            }
            sprintf (Output, "X = %d; Y = %d", cursor_x, cursor_y);
            WriteLn (Output);
            GLCD_Move_Cursor(cursor_x, cursor_y);
          }
          TouchScreenFlag = 1;
      }
      else
      {
        if (TouchScreenFlag)
        {
          WriteLn ("Screen untouched!");
          PrintGlobalPath ();
          TouchScreenFlag = 0;
        }
      }
  }
}


